

#include "WConsole.h"



/////////////////////////////////////////////////////////////////////////////
//
//
//
// Class WConsole
//
//
//
//

WConsole::WConsole()
{
	// Assign a handle to standard out
	m_hConsoleOutput = ::GetStdHandle(STD_OUTPUT_HANDLE);
	
	// Assign a handle to standard in
	m_hConsoleInput = ::GetStdHandle(STD_INPUT_HANDLE);
	
	// get information about the current console
	::GetConsoleCursorInfo(m_hConsoleOutput, &m_NormalCursorInfo);

	// Clear the last error code variable
	m_nLastErrorCode = 0;


	// Enable the window and mouse input events. 
    DWORD fdwMode = ENABLE_WINDOW_INPUT | ENABLE_MOUSE_INPUT; 
    ::SetConsoleMode(m_hConsoleInput, fdwMode);
        

	UpdateConsoleInfo();
}


WConsole::~WConsole()
{
}


HANDLE WConsole::GetOutConsoleHandle()
{
	return m_hConsoleOutput;
}


WConsole& WConsole::Instance()
//
// This function implements the Singleton Design Pattern for this class.  In C++
// the proper way to do this is get the instance of the class from a function
// that has exactly 2 lines.  Line 1 declares the object, Line 2 returns it
// as a pointer or reference.  I prefer references since they are safer than pointers.
{
	// -----------------------------------------------------------------------
	// Single Instance of this class - Note 1 instance total, not per class
	// -----------------------------------------------------------------------
	static WConsole m_SingletonInstance;
	
	// return instance of the singleton
	return m_SingletonInstance;
}



VOID WConsole::SetConsoleCursor(UINT cursorStyle)
{
	CONSOLE_CURSOR_INFO cursInfo;

	switch (cursorStyle)
	{
		case CURSOR_OFF:
		{
			cursInfo.dwSize = m_NormalCursorInfo.dwSize;
			cursInfo.bVisible = FALSE;
		    ::SetConsoleCursorInfo(m_hConsoleOutput, &cursInfo);
		} 
		break;
		case CURSOR_BIG:
		{
			cursInfo.dwSize = 100;
		    cursInfo.bVisible = TRUE;
		    ::SetConsoleCursorInfo(m_hConsoleOutput, &cursInfo);
		} 
		break;
		case CURSOR_NORMAL:
		default:
		{
			::SetConsoleCursorInfo(m_hConsoleOutput, &m_NormalCursorInfo);
		}
	} 
	m_nLastErrorCode = 0;
}



WKey WConsole::GetChar(BOOL bUseBlockingMode)
{
	INPUT_RECORD	ir;
	DWORD			cb;
	WKey			key(0, 0);

	m_nLastErrorCode = 0;

	while (TRUE)
	{

		if (! ReadConsoleInput( 
		            m_hConsoleInput,      // input buffer handle 
			        &ir,     // buffer to read into 
				    1,         // size of read buffer 
					&cb) ) // number of records read 
		{
			m_nLastErrorCode = ::GetLastError();
			return key;
		}

		switch(ir.EventType)
		{
		
		case KEY_EVENT:
		
			if (ir.Event.KeyEvent.bKeyDown == TRUE)
			{
				// Note, UNICODE support requires a change here... 		
				CHAR ch = ir.Event.KeyEvent.uChar.AsciiChar;
				char szTemp[255];
#ifdef _DEBUG
				wsprintf(szTemp, "char=%d, scan=%d, key=%d\n", 
						ir.Event.KeyEvent.uChar.AsciiChar,
						ir.Event.KeyEvent.wVirtualScanCode,
						ir.Event.KeyEvent.wVirtualKeyCode);
				
				::OutputDebugString(szTemp);
#endif
				key.SetChar((CHAR) ch);
				key.SetScanCode(0);
				if (ch == 0)
				{
					UINT sc = ir.Event.KeyEvent.wVirtualScanCode;
					// Since WIN32 returns codes for a control key (i.e. SHIFT) being
					// pressed, and we get this information also on all other key presses
					// we can strip out these codes. 
					if (!((sc==42) || (sc==29) || (sc==56) || (sc==54) || (sc==58) || 
						  (sc==91) || (sc==92) || (sc==93) || (sc==69) || (sc==70)))
					{
						key.SetScanCode(sc);
						return key;
					}
				}
				else
				{
					key.SetScanCode(0);
					return key;
				}

			}
			break;
		
		case MOUSE_EVENT:
			
			if (ir.Event.MouseEvent.dwEventFlags == DOUBLE_CLICK)
			{
				DWORD	dwNumRead	= 0;
				CHAR	chCharRead	= '\0';
				
				BOOL bOk =	ReadConsoleOutputCharacter(m_hConsoleOutput,	// Handle to output window
								&chCharRead,								// character buffer
								1,											// number of cells to read
								ir.Event.MouseEvent.dwMousePosition,        // coordinates of first cell
								&dwNumRead									// number of cells read
							);

				// If all way ok.. Return the char
				if ((bOk) && (dwNumRead>0))
				{
					key.SetChar((CHAR) chCharRead);
					key.SetScanCode(0);
					return key;
				}

			}
			break;
		
		default:
			break;
		
		}
		
		if (!bUseBlockingMode)
		{
			key.SetChar(0);
			key.SetScanCode(0);
			return key;
		}

		Sleep(0);
	}


	key.SetChar(0);
	key.SetScanCode(0);
	return key;
}



VOID WConsole::SetConsoleTitle(LPCTSTR pszNewTitle)
{
	_ASSERT(pszNewTitle);

	m_nLastErrorCode = 0;

	::SetConsoleTitle(pszNewTitle);
}


