// WColorAttribute.h: interface for the WColorAttribute class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_WCOLORATTRIBUTE_H__AB2BE4C4_9A6A_11D3_8104_005004D580FD__INCLUDED_)
#define AFX_WCOLORATTRIBUTE_H__AB2BE4C4_9A6A_11D3_8104_005004D580FD__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include <windows.h>
#include <tchar.h>
#include <crtdbg.h>		// MSCRT Debug Support (_ASSERT macro)

class WColorAttribute  
{
private:

	BYTE m_byBgColor;
	BYTE m_byFgColor;
	BOOL m_bBlink;

public:

	//
	// Color Constants
	//
	static const BYTE	BLACK			;
	static const BYTE	BLUE			;
	static const BYTE	GREEN			;
	static const BYTE	CYAN			;
	static const BYTE	RED				;
	static const BYTE	MAGENTA			;
	static const BYTE	BROWN			;
	static const BYTE	LIGHTGRAY		;
	static const BYTE	DARKGRAY		;
	static const BYTE	LIGHTBLUE		;
	static const BYTE	LIGHTGREEN		;
	static const BYTE	LIGHTCYAN		;
	static const BYTE	LIGHTRED		;
	static const BYTE	LIGHTMAGENTA	;
	static const BYTE	YELLOW			;
	static const BYTE	WHITE			;
	static const BYTE	BLINK			;
	static const BYTE	NORMAL_COLOR	;


	WColorAttribute(BYTE byFgColor=7, BYTE byBgColor=0, BOOL bBlink=FALSE)
	{
		m_byFgColor = byFgColor;
		m_byBgColor = byBgColor;
		m_bBlink    = bBlink;
	}

	virtual ~WColorAttribute();
	
	BYTE GetFgColor()
	{
		return m_byFgColor;
	}

	VOID SetFgColor(BYTE byFgColor)
	{
		m_byFgColor = byFgColor;
	}

	BYTE GetBgColor()
	{
		return m_byBgColor;
	}

	VOID SetBgColor(BYTE byBgColor)
	{
		m_byBgColor = byBgColor;
	}
	
	BOOL IsBlinking()
	{
		return m_bBlink;
	}

	VOID SetBlinking(BOOL bBlink)
	{
		m_bBlink = bBlink;
	}

	BYTE GetAttribute()
	{
		BYTE byAttribute = (((m_bBlink==TRUE) ? 128 : 0) | (m_byBgColor<<4) | (m_byFgColor));

		return byAttribute;
	}

	VOID SetAttribute(BYTE byAttribute)
	{
		m_bBlink = ((byAttribute & 128) ? TRUE : FALSE);
		m_byBgColor = ((byAttribute & 112) >> 4);
		m_byFgColor = (byAttribute & 15);
	}

	operator BYTE() 
	{
		return (BYTE) GetAttribute();
	}


// ----------------------------------------------------------------------------
// Function:    GetAttributeAsAnsiString
// Description: Returns a String representation of the ANSI control sequence for this color code
// Returns:     VOID 
// Parameters:
//		1) LPSTR lpszAnsiAttributeString [OUT] - Ansi String
//
// Notes: 
//			
// 
// ----------------------------------------------------------------------------
	VOID GetAttributeAsAnsiString(LPSTR lpszAnsiAttributeString)
	{
		WColorAttribute::CreateAnsiStringFromAttribute((BYTE)this->GetAttribute(), lpszAnsiAttributeString);
	}


// ----------------------------------------------------------------------------
// Function:    CreateAnsiStringFromAttribute
// Description: Converts a (BYTE) Attribute into a ANSI escape code string.
// Returns:     VOID
// Parameters:
//		1) BYTE byAttribute
//		2) LPSTR lpszAnsiAttributeString
//
// Notes: 
//			Static Function.
// 
// ----------------------------------------------------------------------------
	static VOID CreateAnsiStringFromAttribute(BYTE byAttribute, LPSTR lpszAnsiAttributeString);
	

private:
	//
	// functions used internally by this class
	//

	static VOID AddToAnsiString(LPTSTR lpszAnsiString, UINT32 i);

};

#endif // !defined(AFX_WCOLORATTRIBUTE_H__AB2BE4C4_9A6A_11D3_8104_005004D580FD__INCLUDED_)
