//
//////////////////////////////////////////////////////////////////////

#include "WWindow.h"


/*
UPPER_LEFT		UPPER_RIGHT		LOWER_LEFT		LOWER_RIGHT		LEFT	RIGHT		TOP		BOTTOM
*/

char BORDERCONSTANTS[7][8] = 
{
	{ ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ' },	// BORDER_NONE
	{ '', '', '', '', '', '', '', '' }, // BORDER_SINGLE
	{ '', '', '', '', '', '', '', '' }, // BORDER_DOUBLE
	{ '', '', '', '', '', '', '', '' }, // BORDER_DBL_TOP
	{ '', '', '', '', '', '', '', '' }, // BORDER_DBL_SIDE
	{ '+', '+', '+', '+', '|', '|', '-', '-' }, // BORDER_ASCII
	{ '', '', '', '', '', '', '', '' }  // BORDER_SOLID
};

const char WWINDOW_BS		= '\b';
const char WWINDOW_TAB		= '\t';
const char WWINDOW_CR		= '\r';
const char WWINDOW_LF		= '\n';
const char WWINDOW_FILLCHAR	= 0x20;
const char WWINDOW_NORMATTR	= 0x07;


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

WWindow::WWindow(WConsole& console, WRect& rect, UINT nBorderStyle)
{
	m_colorAttribute.SetBgColor(WColorAttribute::BLACK);
	m_colorAttribute.SetFgColor(WColorAttribute::LIGHTGRAY);

	// Clear the last error code variable
	m_nLastErrorCode = 0;

	// Assign our reference to the Console
	m_pConsole		= &console;

	// Assign a handle to standard out
	m_hConsoleOutput = ::GetStdHandle(STD_OUTPUT_HANDLE);

	// Assign the border style to use for this window
	m_nBorderStyle = nBorderStyle;

	m_rect = rect;

	if (m_nBorderStyle > 0)
	{
		m_rect.Shrink(1);
	}

	m_Curpos.X = m_rect.GetStartX();
	m_Curpos.Y = m_rect.GetStartY();
}


WWindow::~WWindow()
{

}



VOID WWindow::WriteChar(TCHAR ch)
{
	DWORD cb			= 0;
	WORD	attr		= WWINDOW_NORMATTR;
	m_nLastErrorCode	= 0;

	if ((ch == WWINDOW_TAB) || (ch == WWINDOW_CR) || (ch == WWINDOW_LF) || (ch == WWINDOW_BS))
	{
		WriteSpecial(ch);
	} 
	else 
	{
		
		if (m_Curpos.X > m_rect.GetEndX())
		{
			m_Curpos.X = m_rect.GetStartX();
			m_Curpos.Y++;
			
			if (m_Curpos.Y > m_rect.GetEndY())
			{
				ScrollWindowUp(1);
				m_Curpos.Y = m_rect.GetEndY();
			}
		}

		if (!::WriteConsoleOutputCharacter(m_hConsoleOutput, &ch, 1, m_Curpos, &cb))
		{
			m_nLastErrorCode = ::GetLastError();
		}

		attr = (WORD) (BYTE) m_colorAttribute;
		::WriteConsoleOutputAttribute(m_hConsoleOutput, &attr, 1, m_Curpos, &cb);
		m_Curpos.X++;
	}

	::SetConsoleCursorPosition(m_hConsoleOutput, m_Curpos);

}



VOID WWindow::WriteSpecial(TCHAR chSpecialChar)
{
	m_nLastErrorCode = 0;
	switch (chSpecialChar)
	{
	case WWINDOW_TAB:
		{
			// %%TODO: Implement this
		}
		break;
	case WWINDOW_BS:
		{
			if (m_Curpos.X > m_rect.GetStartX())
			{
				m_Curpos.X--;
			}
		}
		break;
	case WWINDOW_CR:
		{
			m_Curpos.X = m_rect.GetStartX();
		}
		break;
	case WWINDOW_LF:
		{
			m_Curpos.X = m_rect.GetStartX();
			m_Curpos.Y++;

			if (m_Curpos.Y > m_rect.GetEndY())
			{
				ScrollWindowUp(1);
				m_Curpos.Y = m_rect.GetEndY();
			}
		}
		break;
	default:
		{
			// Do nothing.
			_ASSERT(FALSE);
		}
		break;
	}
}


VOID WWindow::WriteString(LPCTSTR pszText)
{
	_ASSERT(pszText);
	m_nLastErrorCode = 0;
	LPTSTR pszTemp = (LPTSTR) pszText;


	while (*pszTemp != NULL)
	{
		WriteChar((TCHAR) *pszTemp);
		pszTemp = CharNext(pszTemp);
	}
}


VOID WWindow::WriteString(WORD wAt, LPCTSTR pszText)
{
	_ASSERT(pszText);

	WColorAttribute curAttr = m_colorAttribute;
	m_colorAttribute = (BYTE) wAt;
	WriteString(pszText);
	m_colorAttribute = curAttr;
}


VOID WWindow::WriteString(WPoint &point, WORD wAt, LPCTSTR pszText)
{
	_ASSERT(pszText);

	WPoint savedPoint(m_Curpos.X, m_Curpos.Y);

	m_Curpos.X = point.GetX();
	m_Curpos.Y = point.GetY();

	WriteString(wAt, pszText);
	
	m_Curpos.X = savedPoint.GetX();
	m_Curpos.Y = savedPoint.GetY();

}


VOID WWindow::WriteChar(WORD wAt, TCHAR ch)
{
	WColorAttribute curAttr = m_colorAttribute;
	m_colorAttribute = (BYTE) wAt;
	WriteChar(ch);
	m_colorAttribute = curAttr;
}


VOID WWindow::WriteChar(WPoint &point, WORD wAt, TCHAR ch)
{
	WPoint savedPoint(m_Curpos.X, m_Curpos.Y);

	m_Curpos.X = point.GetX();
	m_Curpos.Y = point.GetY();

	WriteChar(wAt, ch);
	
	m_Curpos.X = savedPoint.GetX();
	m_Curpos.Y = savedPoint.GetY();

}




VOID WWindow::ClearScreen(WORD wAt, CHAR ch)
{
	DWORD cb;
	UINT nLength;
	COORD coord;
	nLength = m_rect.GetWidth();
	for (int i=0; i<m_rect.GetLength(); i++)
	{
		coord.X = m_rect.GetStartX();
		coord.Y = m_rect.GetStartY()+i;
		::FillConsoleOutputCharacter(m_hConsoleOutput, WWINDOW_FILLCHAR, nLength, coord, &cb);
		::FillConsoleOutputAttribute(m_hConsoleOutput, WWINDOW_NORMATTR, nLength, coord, &cb);
	}
	SetScreenLocation(m_rect.GetStartPoint());
}


BOOL WWindow::GetScreenLocation(WPoint &point)
{
	point.SetX(m_Curpos.X);
	point.SetY(m_Curpos.Y);
	m_nLastErrorCode = 0;
	return TRUE;
}


UINT WWindow::GetX()
{
	m_nLastErrorCode = 0;
	return m_Curpos.X;
}


UINT WWindow::GetY()
{
	m_nLastErrorCode = 0;
	return m_Curpos.Y;
}


VOID WWindow::SetScreenLocation(WPoint &point)
{
	// rember that we use 0 based locations
	m_Curpos.X = point.GetX();
	m_Curpos.Y = point.GetY();
	::SetConsoleCursorPosition(m_hConsoleOutput, m_Curpos);
	m_nLastErrorCode = 0;
}


VOID WWindow::ScrollWindowUp(UINT nNumLines)
{
	SMALL_RECT						rectMoveArea;
	COORD							coordDest;
	CHAR_INFO						fillAttribute;

	rectMoveArea.Top				= m_rect.GetStartY()+nNumLines;
	rectMoveArea.Left				= m_rect.GetStartX();
	rectMoveArea.Right				= m_rect.GetEndX();
	rectMoveArea.Bottom				= m_rect.GetEndY();

	fillAttribute.Attributes		= (WORD) (BYTE) m_colorAttribute;
	fillAttribute.Char.AsciiChar	= WWINDOW_FILLCHAR;		// Space
	
	coordDest.X						= m_rect.GetStartX();
	coordDest.Y						= m_rect.GetStartY();

    ::ScrollConsoleScreenBuffer(m_hConsoleOutput, &rectMoveArea, NULL, coordDest, &fillAttribute);

}


VOID WWindow::ScrollWindowDown(UINT nNumLines)
{
	SMALL_RECT						rectMoveArea;
	COORD							coordDest;
	CHAR_INFO						fillAttribute;

	rectMoveArea.Top				= m_rect.GetStartY();
	rectMoveArea.Left				= m_rect.GetStartX();
	rectMoveArea.Right				= m_rect.GetEndX();
	rectMoveArea.Bottom				= m_rect.GetEndY()-nNumLines;

	fillAttribute.Attributes		= (WORD) (BYTE) m_colorAttribute;
	fillAttribute.Char.AsciiChar	= WWINDOW_FILLCHAR;		// Space
	
	coordDest.X						= m_rect.GetStartX();
	coordDest.Y						= m_rect.GetStartY()+nNumLines;

    ::ScrollConsoleScreenBuffer(m_hConsoleOutput, &rectMoveArea, NULL, coordDest, &fillAttribute);

}


VOID WWindow::ScrollWindowLeft(UINT nNumLines)
{
	SMALL_RECT						rectMoveArea;
	COORD							coordDest;
	CHAR_INFO						fillAttribute;

	rectMoveArea.Top				= m_rect.GetStartY();
	rectMoveArea.Left				= m_rect.GetStartX()+nNumLines;
	rectMoveArea.Right				= m_rect.GetEndX();
	rectMoveArea.Bottom				= m_rect.GetEndY();

	fillAttribute.Attributes		= (WORD) (BYTE) m_colorAttribute;
	fillAttribute.Char.AsciiChar	= WWINDOW_FILLCHAR;		// Space
	
	coordDest.X						= m_rect.GetStartX();
	coordDest.Y						= m_rect.GetStartY();

    ::ScrollConsoleScreenBuffer(m_hConsoleOutput, &rectMoveArea, NULL, coordDest, &fillAttribute);

}



VOID WWindow::ScrollWindowRight(UINT nNumLines)
{
	SMALL_RECT						rectMoveArea;
	COORD							coordDest;
	CHAR_INFO						fillAttribute;

	rectMoveArea.Top				= m_rect.GetStartY();
	rectMoveArea.Left				= m_rect.GetStartX();
	rectMoveArea.Right				= m_rect.GetEndX()-nNumLines;
	rectMoveArea.Bottom				= m_rect.GetEndY();

	fillAttribute.Attributes		= (WORD) (BYTE) m_colorAttribute;
	fillAttribute.Char.AsciiChar	= WWINDOW_FILLCHAR;		// Space
	
	coordDest.X						= m_rect.GetStartX()+nNumLines;
	coordDest.Y						= m_rect.GetStartY();

    ::ScrollConsoleScreenBuffer(m_hConsoleOutput, &rectMoveArea, NULL, coordDest, &fillAttribute);

}


// Higher level Text Mode GUI 
VOID WWindow::FillRect(WRect &rect, WORD wAt)
{
}


VOID WWindow::DrawFrame(WORD wAt)
{
}




