// WWindow.h: interface for the WWindow class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_WWINDOW_H__AB2BE4C0_9A6A_11D3_8104_005004D580FD__INCLUDED_)
#define AFX_WWINDOW_H__AB2BE4C0_9A6A_11D3_8104_005004D580FD__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <windows.h>
#include <tchar.h>
#include <crtdbg.h>		// MSCRT Debug Support (_ASSERT macro)

#include "WConsole.h"
#include "WColorAttribute.h"
#include "WKey.h"
#include "WPoint.h"
#include "WRect.h"


class WWindow  
{
public:

// ----------------------------------------------------------------------------
// Output Console Handle
// ----------------------------------------------------------------------------
	HANDLE m_hConsoleOutput;

// -----------------------------------------------------------------------
// Information about the screen
// -----------------------------------------------------------------------
	CONSOLE_SCREEN_BUFFER_INFO m_consoleScreenBufferInfo;


// ----------------------------------------------------------------------------
// Number of Last Error Code from Class
// ----------------------------------------------------------------------------
	DWORD32	m_nLastErrorCode;



// ----------------------------------------------------------------------------
// Current Cursor Location - Note: Box routines do not update this.
// ----------------------------------------------------------------------------
	COORD	m_Curpos;

// ----------------------------------------------------------------------------
// Current Window Size
// ----------------------------------------------------------------------------
	WRect	m_rect;


// -----------------------------------------------------------------------
// Current Color
// -----------------------------------------------------------------------
	WColorAttribute	m_colorAttribute;


// -----------------------------------------------------------------------
// Border Style
// -----------------------------------------------------------------------
	UINT m_nBorderStyle;

// -----------------------------------------------------------------------
// Console
// -----------------------------------------------------------------------
	WConsole *m_pConsole;

public:
	//
	// Box Style Constants
	//
	enum 
	{  
			BORDER_NONE		= 0,
			BORDER_SINGLE	= 1,
			BORDER_DOUBLE	= 2,
			BORDER_DBL_TOP	= 3,
			BORDER_DBL_SIDE = 4,
			BORDER_SOLID	= 5
	};


	//
	// Location to the border characters
	//
	enum
	{
			UPPER_LEFT		= 0,
			UPPER_RIGHT		= 1,
			LOWER_LEFT		= 2,
			LOWER_RIGHT		= 3,
			LEFT			= 4,
			RIGHT			= 5,
			TOP				= 6,
			BOTTOM			= 7
	};


	//
	// Special Character Constants
	//
	enum 
	{	
			CHAR_BS			= 0,
			CHAR_CR			= 1,
			CHAR_LF			= 2,
			CHAR_TAB		= 3
	};


public:
	WWindow(WConsole& console, WRect& rect, UINT nBorderStyle = BORDER_NONE);
	virtual ~WWindow();

	// Lower level API calls
	VOID WriteChar(TCHAR ch);
	VOID WriteChar(WORD wAt, TCHAR ch);
	VOID WriteChar(WPoint &point, WORD wAt, TCHAR ch);
	
	// Uses enum's defined in here
	VOID WriteSpecial(TCHAR chSpecialChar);

	VOID WriteString(LPCTSTR pszText);
	VOID WriteString(WORD wAt, LPCTSTR pszText);
	VOID WriteString(WPoint &point, WORD wAt, LPCTSTR pszText);

	VOID ScrollWindowUp(UINT nNumLines=1);
	VOID ScrollWindowDown(UINT nNumLines=1);
	VOID ScrollWindowLeft(UINT nNumLines=1);
	VOID ScrollWindowRight(UINT nNumLines=1);

	VOID ClearScreen(WORD wAt=WColorAttribute::NORMAL_COLOR, TCHAR chValue=_T(' '));
	BOOL GetScreenLocation(WPoint &point);
	UINT32 GetX();
	UINT32 GetY();
	VOID SetScreenLocation(WPoint &point);

	// Higher level Text Mode GUI 
    VOID FillRect(WRect &rect, WORD wAt);

protected:

    VOID DrawFrame(WORD wAt);


};

#endif // !defined(AFX_WWINDOW_H__AB2BE4C0_9A6A_11D3_8104_005004D580FD__INCLUDED_)
